from tomli._parser import Flags


def test_set_for_relative_key():
    flags = Flags()
    head_key = ("a", "b")
    rel_key = ("c", "d")
    flags.set_for_relative_key(head_key, rel_key, Flags.EXPLICIT_NEST)
    assert not flags.is_(("a",), flags.EXPLICIT_NEST)
    assert not flags.is_(("a", "b"), flags.EXPLICIT_NEST)
    assert flags.is_(("a", "b", "c"), flags.EXPLICIT_NEST)
    assert flags.is_(("a", "b", "c", "d"), flags.EXPLICIT_NEST)
    assert not flags.is_(("a", "b", "c", "d", "e"), flags.EXPLICIT_NEST)



import copy
import datetime
from decimal import Decimal as D
from pathlib import Path

import pytest

import tomli


def test_load(tmp_path):
    content = "one=1 \n two='two' \n arr=[]"
    expected = {"one": 1, "two": "two", "arr": []}
    file_path = tmp_path / "test.toml"
    file_path.write_text(content)

    # Test text mode
    with open(file_path, encoding="utf-8", newline="") as f:
        with pytest.warns(DeprecationWarning):
            actual = tomli.load(f)  # type: ignore[arg-type]
    assert actual == expected

    # Test binary mode
    with open(file_path, "rb") as bin_f:
        actual = tomli.load(bin_f)
    assert actual == expected


def test_parse_float():
    doc = """
          val=0.1
          biggest1=inf
          biggest2=+inf
          smallest=-inf
          notnum1=nan
          notnum2=-nan
          notnum3=+nan
          """
    obj = tomli.loads(doc, parse_float=D)
    expected = {
        "val": D("0.1"),
        "biggest1": D("inf"),
        "biggest2": D("inf"),
        "smallest": D("-inf"),
        "notnum1": D("nan"),
        "notnum2": D("-nan"),
        "notnum3": D("nan"),
    }
    for k, expected_val in expected.items():
        actual_val = obj[k]
        assert isinstance(actual_val, D)
        if actual_val.is_nan():
            assert expected_val.is_nan()
        else:
            assert actual_val == expected_val


def test_deepcopy():
    doc = """
          [bliibaa.diibaa]
          offsettime=[1979-05-27T00:32:00.999999-07:00]
          """
    obj = tomli.loads(doc)
    obj_copy = copy.deepcopy(obj)
    assert obj_copy == obj
    expected_obj = {
        "bliibaa": {
            "diibaa": {
                "offsettime": [
                    datetime.datetime(
                        1979,
                        5,
                        27,
                        0,
                        32,
                        0,
                        999999,
                        tzinfo=datetime.timezone(datetime.timedelta(hours=-7)),
                    )
                ]
            }
        }
    }
    assert obj_copy == expected_obj


def test_own_pyproject():
    pyproject_path = Path(__file__).parent.parent / "pyproject.toml"
    with open(pyproject_path, "rb") as f:
        pyproject = tomli.load(f)
    assert pyproject["project"]["version"] == tomli.__version__



import json
from pathlib import Path

import pytest

import tomli
from . import burntsushi

DATA_DIR = Path(__file__).parent / "data" / "extras"

VALID_FILES = tuple((DATA_DIR / "valid").glob("**/*.toml"))
VALID_FILES_EXPECTED = tuple(
    json.loads(p.with_suffix(".json").read_bytes().decode()) for p in VALID_FILES
)

INVALID_FILES = tuple((DATA_DIR / "invalid").glob("**/*.toml"))


@pytest.mark.parametrize(
    "invalid",
    INVALID_FILES,
    ids=[p.stem for p in INVALID_FILES],
)
def test_invalid(invalid):
    toml_str = invalid.read_bytes().decode()
    with pytest.raises(tomli.TOMLDecodeError):
        tomli.loads(toml_str)


@pytest.mark.parametrize(
    "valid,expected",
    zip(VALID_FILES, VALID_FILES_EXPECTED),
    ids=[p.stem for p in VALID_FILES],
)
def test_valid(valid, expected):
    toml_str = valid.read_bytes().decode()
    actual = tomli.loads(toml_str)
    actual = burntsushi.convert(actual)
    expected = burntsushi.normalize(expected)
    assert actual == expected



import json
from pathlib import Path

import pytest

import tomli
from . import burntsushi


class MissingFile:
    def __init__(self, path: Path):
        self.path = path


DATA_DIR = Path(__file__).parent / "data" / "toml-lang-compliance"

VALID_FILES = tuple((DATA_DIR / "valid").glob("**/*.toml"))
# VALID_FILES_EXPECTED = tuple(
#     json.loads(p.with_suffix(".json").read_bytes().decode()) for p in VALID_FILES
# )
_expected_files = []
for p in VALID_FILES:
    json_path = p.with_suffix(".json")
    try:
        text = json.loads(json_path.read_bytes().decode())
    except FileNotFoundError:
        text = MissingFile(json_path)
    _expected_files.append(text)

VALID_FILES_EXPECTED = tuple(_expected_files)
INVALID_FILES = tuple((DATA_DIR / "invalid").glob("**/*.toml"))


@pytest.mark.parametrize(
    "invalid",
    INVALID_FILES,
    ids=[p.stem for p in INVALID_FILES],
)
def test_invalid(invalid):
    toml_str = invalid.read_bytes().decode()
    with pytest.raises(tomli.TOMLDecodeError):
        tomli.loads(toml_str)


@pytest.mark.parametrize(
    "valid,expected",
    zip(VALID_FILES, VALID_FILES_EXPECTED),
    ids=[p.stem for p in VALID_FILES],
)
def test_valid(valid, expected):
    if isinstance(expected, MissingFile):
        pytest.xfail(f"Missing a .json file corresponding the .toml: {expected.path}")
    toml_str = valid.read_bytes().decode()
    actual = tomli.loads(toml_str)
    actual = burntsushi.convert(actual)
    expected = burntsushi.normalize(expected)
    assert actual == expected



import pytest

import tomli


def test_line_and_col():
    with pytest.raises(tomli.TOMLDecodeError) as exc_info:
        tomli.loads("val=.")
    assert str(exc_info.value) == "Invalid value (at line 1, column 5)"

    with pytest.raises(tomli.TOMLDecodeError) as exc_info:
        tomli.loads(".")
    assert str(exc_info.value) == "Invalid statement (at line 1, column 1)"

    with pytest.raises(tomli.TOMLDecodeError) as exc_info:
        tomli.loads("\n\nval=.")
    assert str(exc_info.value) == "Invalid value (at line 3, column 5)"

    with pytest.raises(tomli.TOMLDecodeError) as exc_info:
        tomli.loads("\n\n.")
    assert str(exc_info.value) == "Invalid statement (at line 3, column 1)"


def test_missing_value():
    with pytest.raises(tomli.TOMLDecodeError) as exc_info:
        tomli.loads("\n\nfwfw=")
    assert str(exc_info.value) == "Invalid value (at end of document)"


def test_invalid_char_quotes():
    with pytest.raises(tomli.TOMLDecodeError) as exc_info:
        tomli.loads("v = '\n'")
    assert " '\\n' " in str(exc_info.value)


def test_module_name():
    assert tomli.TOMLDecodeError().__module__ == "tomli"



"""Utilities for tests that are in the "burntsushi" format."""

import datetime
from typing import Any

import dateutil.parser
import pytest


def convert(obj):  # noqa: C901
    if isinstance(obj, str):
        return {"type": "string", "value": obj}
    elif isinstance(obj, bool):
        return {"type": "boolean", "value": str(obj).lower()}
    elif isinstance(obj, int):
        return {"type": "integer", "value": str(obj)}
    elif isinstance(obj, float):
        return {"type": "float", "value": str(obj)}
    elif isinstance(obj, datetime.datetime):
        val = normalize_datetime_str(obj.isoformat())
        if obj.tzinfo:
            return {"type": "offset datetime", "value": val}
        return {"type": "local datetime", "value": val}
    elif isinstance(obj, datetime.time):
        return {
            "type": "local time",
            "value": str(obj),
        }
    elif isinstance(obj, datetime.date):
        return {
            "type": "local date",
            "value": str(obj),
        }
    elif isinstance(obj, list):
        return {
            "type": "array",
            "value": [convert(i) for i in obj],
        }
    elif isinstance(obj, dict):
        return {k: convert(v) for k, v in obj.items()}
    raise Exception("unsupported type")


def normalize(d: dict) -> dict:
    normalized: Any = {}
    for k, v in d.items():
        if isinstance(v, list):
            normalized[k] = [normalize(item) for item in v]
        elif isinstance(v, dict):
            if "type" in v and "value" in v:
                if v["type"] == "float":
                    normalized[k] = v.copy()
                    normalized[k]["value"] = normalize_float_str(normalized[k]["value"])
                elif v["type"] in {"offset datetime", "local datetime"}:
                    normalized[k] = v.copy()
                    normalized[k]["value"] = normalize_datetime_str(
                        normalized[k]["value"]
                    )
                else:
                    normalized[k] = v
            else:
                normalized[k] = v
        else:
            pytest.fail("Burntsushi fixtures should be dicts/lists only")
    return normalized


def normalize_datetime_str(dt_str: str) -> str:
    return dateutil.parser.isoparse(dt_str).isoformat()


def normalize_float_str(float_str: str) -> str:
    return str(float(float_str))
